/*
 * Svjatoslav Commons - shared library of common functionality. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.commons.data;


import java.io.IOException;
import java.io.InputStream;

/**
 * Read individual bits from the input stream.
 */
public class BitInputStream {

    private final InputStream inputStream;
    private int currentByte;
    private int currentBytePointer = -1;

    public BitInputStream(final InputStream inputStream) {
        this.inputStream = inputStream;
    }

    public int readBits(final int bitCount) throws IOException {

        int readableByte = 0;
        for (int i = 0; i < bitCount; i++) {

            readableByte = readableByte << 1;

            if (currentBytePointer == -1) {
                currentBytePointer = 7;
                currentByte = inputStream.read();
            }

            int mask = 1;
            mask = mask << currentBytePointer;

            final int currentBit = currentByte & mask;

            if (currentBit != 0)
                readableByte = readableByte | 1;

            currentBytePointer--;
        }
        return readableByte;
    }

}
