/*
 * Svjatoslav Commons - shared library of common functionality. Author: Svjatoslav Agejenko.
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.commons.gui.dialog;

import javax.swing.*;
import java.awt.*;

import static javax.swing.BoxLayout.Y_AXIS;

public class ExceptionDialog {

    public ExceptionDialog(final Exception exception) {
        showException(exception);
    }

    /**
     * This method is for testing
     *
     * @param args commandline arguments
     */
    public static void main(final String[] args) {

        final Throwable cause = new Throwable("details.....");

        final Exception exception = new Exception("test", cause);

        new ExceptionDialog(exception);
    }

    /**
     * @param exception exception to show
     */
    public void showException(final Exception exception) {
        final JPanel contentPanel = new JPanel(new BorderLayout());
        contentPanel.add(getTopPanel(exception), BorderLayout.NORTH);
        contentPanel.add(getStackTraceView(exception.getCause()), BorderLayout.CENTER);

        final JFrame frame = new JFrame("Exception occurred!");
        frame.getContentPane().add(contentPanel);
        frame.setSize(800, 600);
        frame.setVisible(true);
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

        centerFrameOnScreen(frame);
    }

    private void centerFrameOnScreen(JFrame frame) {
        final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setLocation(
                (screenSize.width - frame.getWidth()) / 2,
                (screenSize.height - frame.getHeight()) / 2);
    }

    private TextArea getStackTraceView(Throwable cause) {
        final StringBuilder buffer = new StringBuilder();

        if (cause != null) {
            // if cause is available, show original stack trace
            buffer.append("Stack trace:\n");
            final StackTraceElement[] stackTrace = cause.getStackTrace();
            enlistStackTraceElements(buffer, stackTrace);
        } else {
            // otherwise show at least current stack trace
            buffer.append("Stack trace from original cause is not available.\n" +
                    "Showing current stack trace instead:\n");
            enlistStackTraceElements(buffer, new Exception("Stack trace").getStackTrace());
        }

        return new TextArea(buffer.toString());
    }

    private void enlistStackTraceElements(StringBuilder buffer, StackTraceElement[] stackTrace) {
        for (final StackTraceElement stackTraceElement : stackTrace)
            buffer.append(stackTraceElement.toString()).append("\n");
    }

    private JPanel getTopPanel(Exception exception) {

        final JPanel topPanel = new JPanel();
        topPanel.setLayout(new BoxLayout(topPanel, Y_AXIS));

        // add exception type
        topPanel.add(
                new JLabel("Exception type: " + exception.getClass().getCanonicalName()));

        // add error message
        topPanel.add(new JLabel("Error message: " + exception.getMessage()));

        // add cause message
        Throwable cause = exception.getCause();
        if (cause != null && cause.getMessage() != null)
            topPanel.add(new JLabel("Cause: " + cause.getMessage()));
        return topPanel;
    }

}
